@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  *
  * Author:  Roberto Majadas <roberto.majadas@openshine.com>
  */
@@ -57,12 +57,11 @@
 #define SAVE_TIMEOUT 5000
 
 
-static 
-guint save_blist_timeout_handler, take_spool_files_handler;
-gboolean taking_files;
-GString *buddies_str;
+static guint take_spool_files_handler;
+static gboolean taking_files;
+static GString *buddies_str;
 
-void
+static void
 get_online_buddies (PurpleBlistNode *node, GString *str){
 
     PurpleBlistNode *aux;
@@ -101,8 +100,8 @@ get_online_buddies (PurpleBlistNode *node, GString *str){
 
 }
 
-gint
-save_online_buddies (){
+static void
+save_online_buddies (PurpleBuddy *buddy, gpointer data){
     PurpleBuddyList *blist;
     GString *str;
     gchar *fd_name;
@@ -117,27 +116,25 @@ save_online_buddies (){
     str = g_string_append (str, "---\n");
 
     if (!g_string_equal (buddies_str, str)){
-	fd = fopen (fd_name, "w");
-	if (fd){
-	    fwrite (str->str, 1, str->len, fd);
-	    fclose (fd);
-	    g_string_free (buddies_str, TRUE);
-	    buddies_str = str;
-	    gaim_debug_info ("nautilus", "save blist online\n");
-	}else{
-	    g_string_free (str, TRUE);
-	    gaim_debug_info ("nautilus", "don't save blist online. No change\n");
-	}
-	g_free (fd_name);
+    	    GError *err = NULL;
+	    if (g_file_set_contents (fd_name, str->str, str->len, &err) == FALSE) {
+	    	    purple_debug_info ("nautilus", "couldn't save '%s': %s\n", fd_name, err->message);
+	    	    g_error_free (err);
+	    	    g_string_free (buddies_str, TRUE);
+	    } else {
+	    	    purple_debug_info ("nautilus", "saved blist online\n");
+		    g_string_free (buddies_str, TRUE);
+		    buddies_str = str;
+	    }
     }else{
 	g_string_free (str, TRUE);
 	purple_debug_info ("nautilus", "don't save blist online. No change\n");
     }
-    
-    return TRUE;
+    g_free (fd_name);
+
 }
 
-void
+static void
 init_plugin_stuff (){
     gchar *plugin_home;
     gchar *spool;
@@ -164,7 +161,7 @@ init_plugin_stuff (){
     g_free (spool_tmp);
 }
 
-void
+static void
 send_file (GString *username, GString *cname,
 	   GString *protocol, GString *file){
 
@@ -178,7 +175,7 @@ send_file (GString *username, GString *cname,
     serv_send_file (account->gc, cname->str, file->str);
 }
 
-void
+static void
 process_file (gchar *file){
     GIOChannel *io;
     GString *username;
@@ -258,18 +255,29 @@ take_spool_files(){
 }
 
 static gboolean
-plugin_load(PurplePlugin *plugin) {
-	
+plugin_load(PurplePlugin *plugin)
+{
+	void *blist_handle;
 	
 	init_plugin_stuff ();
 	buddies_str = g_string_new ("");
-	save_blist_timeout_handler = purple_timeout_add (5000,
-						       save_online_buddies,
-						       NULL);
+
+	blist_handle = purple_blist_get_handle();
+
+	purple_signal_connect (blist_handle, "buddy-signed-on",
+			       plugin, (PurpleCallback) save_online_buddies,
+			       NULL);
+	purple_signal_connect (blist_handle, "buddy-signed-off",
+			       plugin, (PurpleCallback) save_online_buddies,
+			       NULL);
 	taking_files = FALSE;
 	take_spool_files_handler = purple_timeout_add (3000,
 						     take_spool_files,
 						     NULL);
+
+	/* And save a list already */
+	save_online_buddies (NULL, NULL);
+
 	return TRUE;
 }
 
@@ -279,9 +287,8 @@ plugin_unload() {
 
     fd_name = g_build_path ("/", g_get_home_dir(), PLUGIN_HOME, 
 			    B_ONLINE, NULL);    
-    purple_timeout_remove (save_blist_timeout_handler);
     purple_timeout_remove (take_spool_files_handler);
-    remove (fd_name);
+    g_unlink (fd_name);
     g_free(fd_name);
     g_string_free (buddies_str, TRUE);
     purple_debug_info ("nautilus", "Stop nautilus plugin\n");
@@ -289,30 +296,46 @@ plugin_unload() {
     return TRUE;
 }
 
+static gboolean
+force_load_once (gpointer data)
+{
+    PurplePlugin *plugin = (PurplePlugin *)data;
+    if (!purple_prefs_get_bool ("/plugins/gtk/nautilus/auto_loaded")) {
+	    purple_debug_info ("nautilus", "Force loading nautilus plugin\n");
+	    purple_plugin_load (plugin);
+		purple_plugins_save_loaded (PIDGIN_PREFS_ROOT "/plugins/loaded");
+	    purple_prefs_set_bool ("/plugins/gtk/nautilus/auto_loaded", TRUE);
+    }
+
+    return FALSE;
+}
+
 static void 
 init_plugin(PurplePlugin *plugin) {
-	
+    purple_prefs_add_none ("/plugins/gtk/nautilus");
+    purple_prefs_add_bool ("/plugins/gtk/nautilus/auto_loaded", FALSE);
+	g_idle_add(force_load_once, plugin);
 }
 
 static PurplePluginInfo info = {
-    PURPLE_PLUGIN_MAGIC,			/* api version */
+    PURPLE_PLUGIN_MAGIC,		/* api version */
     PURPLE_MAJOR_VERSION,
     PURPLE_MINOR_VERSION,
     PURPLE_PLUGIN_STANDARD,		/* type */
-    PIDGIN_PLUGIN_TYPE,		/* ui requirement */
-    0,					/* flags */
+    PIDGIN_PLUGIN_TYPE,			/* ui requirement */
+    PURPLE_PRIORITY_DEFAULT,		/* flags */
     NULL,				/* dependencies */
     PURPLE_PRIORITY_DEFAULT,		/* priority */
-    
-    "gtk-nautilus",					/* id */
-    N_("Nautilus Integration"),				/* name */
-    "0.8",						/* version */
+
+    "gtk-nautilus",			/* id */
+    N_("Nautilus Integration"),		/* name */
+    "0.8",				/* version */
     N_("Provides integration with Nautilus"),		/* summary */ 
     N_("Provides integration with Nautilus"),		/* description */
-    
+
     "Roberto Majadas <roberto.majadas@openshine.com>",	/* author */
     "www.gnome.org",                    /* homepage */
-    
+
     plugin_load,			/* load */
     plugin_unload,			/* unload */
     NULL,				/* destroy */
