@@ -35,9 +35,10 @@
 #define NAUTILUS_SENDTO_GCONF		"/desktop/gnome/nautilus-sendto"
 #define NAUTILUS_SENDTO_LAST_MEDIUM	NAUTILUS_SENDTO_GCONF"/last_medium"
 #define NAUTILUS_SENDTO_LAST_COMPRESS	NAUTILUS_SENDTO_GCONF"/last_compress"
-#define NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT 10000
+#define NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT_SECONDS 10
 
-#define UNINSTALLED_PLUGINDIR "plugins/.libs"
+#define UNINSTALLED_PLUGINDIR "plugins/sylpheed-claws"
+#define UNINSTALLED_SOURCE "nautilus-sendto-command.c"
 
 /* Options */
 static gchar *default_url = NULL;
@@ -286,9 +287,9 @@ send_button_cb (GtkWidget *widget, gpointer data)
 			g_free (error);
 			gtk_label_set_markup (GTK_LABEL (ui->status_label), message);
 			g_free (message);
-			ui->status_timeoutid = g_timeout_add (NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT,
-							      status_label_clear,
-							      ui);
+			ui->status_timeoutid = g_timeout_add_seconds (NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT_SECONDS,
+								      status_label_clear,
+								      ui);
 			gtk_widget_show (ui->status_image);
 			gtk_widget_show (ui->status_box);
 			gtk_widget_set_sensitive (ui->dialog, TRUE);
@@ -410,8 +411,8 @@ set_contact_widgets (NS_ui *ui){
 		gtk_widget_hide (GTK_WIDGET(w));
 		ui->contact_widgets = g_list_append (ui->contact_widgets, w);
 		if (GTK_IS_ENTRY (w)) {
-			g_signal_connect (G_OBJECT (w), "activate",
-					G_CALLBACK (send_if_no_pack_cb), ui);
+			g_signal_connect_after (G_OBJECT (w), "activate",
+						G_CALLBACK (send_if_no_pack_cb), ui);
 		}
 	}
 }
@@ -582,20 +583,14 @@ nautilus_sendto_create_ui (void)
 
 }
 
-static gboolean
-nautilus_sendto_plugin_init (void)
+static void
+nautilus_sendto_plugin_dir_process (const char *plugindir)
 {
 	GDir *dir;
 	const char *item;
 	NstPlugin *p = NULL;
 	gboolean (*nst_init_plugin)(NstPlugin *p);
 	GError *err = NULL;
-	const char *plugindir;
-
-	if (g_file_test (UNINSTALLED_PLUGINDIR, G_FILE_TEST_IS_DIR) != FALSE)
-		plugindir = UNINSTALLED_PLUGINDIR;
-	else
-		plugindir = PLUGINDIR;
 
 	dir = g_dir_open (plugindir, 0, &err);
 
@@ -603,7 +598,6 @@ nautilus_sendto_plugin_init (void)
 		g_warning ("Can't open the plugins dir: %s", err ? err->message : "No reason");
 		if (err)
 			g_error_free (err);
-		return FALSE;
 	} else {
 		while ((item = g_dir_read_name(dir))) {
 			if (g_str_has_suffix (item, SOEXT)) {
@@ -637,6 +631,38 @@ nautilus_sendto_plugin_init (void)
 		}
 		g_dir_close (dir);
 	}
+}
+
+static gboolean
+nautilus_sendto_plugin_init (void)
+{
+	if (g_file_test (UNINSTALLED_PLUGINDIR, G_FILE_TEST_IS_DIR) != FALSE) {
+		/* Try to load the local plugins */
+		GError *err = NULL;
+		GDir *dir;
+		const char *item;
+
+		dir = g_dir_open ("plugins/", 0, &err);
+		if (dir == NULL) {
+			g_warning ("Can't open the plugins dir: %s", err ? err->message : "No reason");
+			if (err)
+				g_error_free (err);
+			return FALSE;
+		}
+		while ((item = g_dir_read_name(dir))) {
+			char *plugindir;
+
+			plugindir = g_strdup_printf ("plugins/%s/.libs/", item);
+			if (g_file_test (plugindir, G_FILE_TEST_IS_DIR) != FALSE)
+				nautilus_sendto_plugin_dir_process (plugindir);
+			g_free (plugindir);
+		}
+		g_dir_close (dir);
+	}
+
+	if (g_list_length (plugin_list) == 0)
+		nautilus_sendto_plugin_dir_process (PLUGINDIR);
+
 	return g_list_length (plugin_list) != 0;
 }
 
@@ -687,44 +713,28 @@ nautilus_sendto_init (void)
 	}
 
 	for (i = 0; filenames != NULL && filenames[i] != NULL; i++) {
-		const char *filename;
- 		char *path;
+		GFile *file;
+		char *filename, *escaped, *uri;
 
-		filename = filenames[i];
- 
- 		if (g_str_has_prefix (filename, "file://")) {
- 			file_list = g_list_prepend (file_list,
- 					g_strdup (filename));
- 			path = g_filename_from_uri (filename, NULL, NULL);
- 			if (path != NULL
- 			    && g_file_test (path, G_FILE_TEST_IS_DIR)) {
- 				force_user_to_compress = TRUE;
- 			}
- 			g_free (path);
- 		} else {
- 			char *uri, *escaped;
+		file = g_file_new_for_commandline_arg (filenames[i]);
+		filename = g_file_get_path (file);
+		if (filename == NULL)
+			continue;
+
+		if (g_file_test (filename, G_FILE_TEST_IS_DIR) != FALSE)
+			force_user_to_compress = TRUE;
+
+		uri = g_filename_to_uri (filename, NULL, NULL);
+		escaped = escape_ampersands (uri);
+		g_free (uri);
+
+		if (escaped == NULL)
+			file_list = g_list_prepend (file_list, filename);
+		else {
+			g_free (filename);
+			file_list = g_list_prepend (file_list, escaped);
+		}
 
- 			if (filename[0] != G_DIR_SEPARATOR) {
- 				path = g_build_filename (default_url,
-						filename, NULL);
-			} else {
-				path = g_strdup (filename);
-			}
- 
- 			uri = g_filename_to_uri (path, NULL, NULL);
-			escaped = escape_ampersands (uri);
-			if (escaped) {
-				file_list = g_list_prepend (file_list, escaped);
-				g_free (uri);
-			} else {
-				file_list = g_list_prepend (file_list, uri);
-			}
- 
- 			if (g_file_test (path, G_FILE_TEST_IS_DIR))
- 				force_user_to_compress = TRUE;
- 
- 			g_free (path);
- 		}
 	}
 
 	if (file_list == NULL) {
